/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2022
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include "ArmarXCore/libraries/ArmarXCoreComponentPlugins/DebugObserverComponentPlugin.h"
#include <ArmarXCore/core/Component.h>

#include <ArmarXGui/libraries/ArmarXGuiComponentPlugins/LightweightRemoteGuiComponentPlugin.h>

#include "RobotAPI/libraries/armem/client/plugins/ReaderWriterPlugin.h"
#include "RobotAPI/libraries/armem_laser_scans/server/Visu.h"
#include "RobotAPI/libraries/armem_robot_state/client/common/VirtualRobotReader.h"
#include <RobotAPI/libraries/RobotAPIComponentPlugins/ArVizComponentPlugin.h>
#include <RobotAPI/libraries/armem/server/plugins/ReadWritePluginUser.h>

namespace armarx::armem::server::laser_scans
{
    /**
 * @defgroup Component-GeneralPurposeMemory GeneralPurposeMemory
 * @ingroup RobotAPI-Components
 * A description of the component GeneralPurposeMemory.
 *SpecializedCoreSegment
 * @class GeneralPurposeMemory
 * @ingroup Component-GeneralPurposeMemory
 * @brief Brief description of class GeneralPurposeMemory.
 *
 * Detailed description of class GeneralPurposeMemory.
 */
    class LaserScansMemory :
        virtual public armarx::Component,
        virtual public armem::server::ReadWritePluginUser,
        virtual public armarx::ArVizComponentPluginUser
    {
    public:
        LaserScansMemory();

        /// @see armarx::ManagedIceObject::getDefaultName()
        std::string getDefaultName() const override;

        armem::data::AddSegmentsResult addSegments(const armem::data::AddSegmentsInput& input,
                                                   const Ice::Current&) override;

    protected:
        /// @see PropertyUser::createPropertyDefinitions()
        armarx::PropertyDefinitionsPtr createPropertyDefinitions() override;

        /// @see armarx::ManagedIceObject::onInitComponent()
        void onInitComponent() override;

        /// @see armarx::ManagedIceObject::onConnectComponent()
        void onConnectComponent() override;

        /// @see armarx::ManagedIceObject::onDisconnectComponent()
        void onDisconnectComponent() override;

        /// @see armarx::ManagedIceObject::onExitComponent()
        void onExitComponent() override;

    private:
        bool addCoreSegmentOnUsage = true;

        armem::client::plugins::ReaderWriterPlugin<armarx::armem::robot_state::VirtualRobotReader>*
            virtualRobotReaderPlugin = nullptr;

        armarx::plugins::DebugObserverComponentPlugin* debugObserver = nullptr;


        Visu commonVisu;
    };
} // namespace armarx::armem::server::laser_scans
