/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::gui-plugins::SkillManagerMonitorWidgetController
 * @author     Raphael Grimm ( raphael dot grimm at kit dot edu )
 * @date       2020
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */
#pragma once

#include "../modal/AronTreeWidgetModal.h"

#include <RobotAPI/libraries/aron/core/type/variant/All.h>
#include <RobotAPI/libraries/aron/core/data/variant/All.h>
#include <RobotAPI/libraries/aron/core/type/visitor/variant/VariantVisitor.h>

namespace armarx
{
    // Convert aron type to tree widget.
    // (Widgets are only created for string types to enter longer texts.
    // However, the visitor implementation allows modals for differnt types. Might be useful in the future..)
    class AronTreeWidgetModalCreatorVisitor :
            public armarx::aron::type::ConstVariantVisitor
    {
    public:
        std::string label = "";
        AronTreeWidgetItem* item = nullptr;
        QTreeWidget* parent = nullptr;
        AronTreeWidgetModalControllerPtr createdModal = nullptr;

        AronTreeWidgetModalCreatorVisitor() = delete;
        AronTreeWidgetModalCreatorVisitor(const std::string& label, AronTreeWidgetItem* item, QTreeWidget* parent) :
            label(label),
            item(item),
            parent(parent)
        {}

        void visitAronVariant(const aron::type::StringPtr& i) final;
        void visitUnknown(Input&) final;
    };
}


