/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::marker_pose_data
 * @author     Hawo Höfer ( uuujt at student dot kit dot edu )
 * @date       2022
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "types.h"

#include "RobotAPI/libraries/core/FramedPose.h"
#include <RobotAPI/libraries/armem_objects/aron/Marker.aron.generated.h>
#include <RobotAPI/libraries/aron/common/aron_conversions/framed.h>

#include "aron_forward_declarations.h"

namespace armarx::armem::marker
{
    Marker::Marker(const std::string& name,
                   const FramedPose& framedMarkerPose,
                   const FramedPose& framedRobotPose,
                   const FramedPose& framedRGBCameraPose,
                   const FramedPose& framedDepthCameraPose) :
        name(name),
        robotGlobal(framedRobotPose),
        rgbCamera(framedRGBCameraPose),
        depthCamera(framedDepthCameraPose),
        markerPose(framedMarkerPose)
    {
        markerGlobal = getGlobalMarkerPose();
    }

    Marker::Marker(const armarx::armem::arondto::Marker& dto) :
        name(dto.name),
        robotGlobal(
            armarx::fromAron<armarx::FramedPose, armarx::arondto::FramedPose>(dto.robotGlobal)),
        rgbCamera(armarx::fromAron<armarx::FramedPose, armarx::arondto::FramedPose>(dto.rgbCamera)),
        depthCamera(
            armarx::fromAron<armarx::FramedPose, armarx::arondto::FramedPose>(dto.depthCamera)),
        markerPose(
            armarx::fromAron<armarx::FramedPose, armarx::arondto::FramedPose>(dto.markerPose))
    {
        markerGlobal = getGlobalMarkerPose();
    }

    armarx::FramedPose
    Marker::getGlobalMarkerPose() const
    {
        return {robotGlobal.toEigen() * depthCamera.toEigen() * markerPose.toEigen(),
                GlobalFrame,
                "marker"};
    }

} // namespace armarx::armem::marker
