#pragma once
#include <experimental/memory>
#include <functional>
#include <queue>
#include <shared_mutex>
#include <thread>
#include <type_traits>

#include <SimoxUtility/json/json.h>

#include <ArmarXCore/core/ComponentPlugin.h>
#include <ArmarXCore/core/ManagedIceObject.h>
#include <ArmarXCore/core/services/tasks/RunningTask.h>

#include <RobotAPI/interface/robot_name_service/RobotNameServiceInterface.h>
#include <RobotAPI/libraries/robot_name_service/core/Robot.h>

namespace armarx::robot_name_service::client::constants
{
    const auto CONFIG_ROBOT_NAME = "name";
    const auto CONFIG_XML_PACKAGE_PATH = "xml_package_path";
    const auto CONFIG_XML_PACKAGE_PATH_PACKAGE = "package";
    const auto CONFIG_XML_PACKAGE_PATH_PATH = "path";
    const auto CONFIG_MNS_PRX = "mns_prx";
    const auto CONFIG_SKILLS_MANAGER_PRX = "manager_prx";
    const auto CONFIG_ARMS = "arms";
    const auto CONFIG_ARM_KINEMATIC_CHAIN_NAME = "kinematic_chain_name";
    const auto CONFIG_ARM_HAND = "hand";
    const auto CONFIG_ARM_HAND_NAME = "name";
    const auto CONFIG_ARM_HAND_FT_NAME = "ft_name";
    const auto CONFIG_ARM_HAND_PRX = "hand_prx";
    // TODO
} // namespace armarx::robot_name_service::client::constants

namespace armarx
{
    class RobotNameServiceComponentPluginUser; // forward declaration
}

namespace armarx::plugins
{
    class RobotNameServiceComponentPlugin : public ComponentPlugin
    {
    public:
        using ComponentPlugin::ComponentPlugin;

        void postOnInitComponent() override;

        void preOnConnectComponent() override;

        void postCreatePropertyDefinitions(PropertyDefinitionsPtr& properties) override;

        void preOnDisconnectComponent() override;

    private:
        struct Properties
        {
            std::string configJson = "{}";

            armarx::robot_name_service::dti::RobotNameServiceInterfacePrx rns;
        } properties;

        armarx::robot_name_service::core::Robot robot;

        friend class armarx::RobotNameServiceComponentPluginUser;
    };
} // namespace armarx::plugins

namespace armarx

{
    class RobotNameServiceComponentPluginUser : virtual public ManagedIceObject
    {
    public:
        RobotNameServiceComponentPluginUser();


    private:
        std::experimental::observer_ptr<plugins::RobotNameServiceComponentPlugin> plugin = nullptr;
    };
} // namespace armarx
