#include "Robot.h"

namespace armarx::robot_name_service::core
{

    void
    Hand::fromIce(const dto::Hand& r)
    {
        name = r.name;
        ft_name = r.ft_name;
        handUnitInterface = r.handUnitInterface;
    }

    dto::Hand
    Hand::toIce() const
    {
        dto::Hand r;
        r.name = name;
        r.ft_name = ft_name;
        r.handUnitInterface = handUnitInterface;
        return r;
    }

    void
    Arm::fromIce(const dto::Arm& r)
    {
        kinematicChainName = r.kinematicChainName;
        hand.fromIce(r.hand);
    }

    dto::Arm
    Arm::toIce() const
    {
        dto::Arm r;
        r.kinematicChainName = kinematicChainName;
        r.hand = hand.toIce();
        return r;
    }

    void
    Robot::fromIce(const dto::Robot& r)
    {
        name = r.name;
        xmlPackagePath = r.xmlPackagePath;
        memoryNameSystem = armarx::armem::client::MemoryNameSystem(r.memoryNameSystem);
        skillManager = r.skillManager;

        arms.clear();
        for (const auto& [k, a] : r.arms)
        {
            arms[k].fromIce(a);
        }

        kinematicUnitInterface = r.kinematicUnitInterface;
        platformUnitInterface = r.platformUnitInterface;
        localizationUnitInterface = r.localizationUnitInterface;
    }

    dto::Robot
    Robot::toIce() const
    {
        dto::Robot r;
        r.name = name;
        r.xmlPackagePath = xmlPackagePath;
        r.memoryNameSystem = memoryNameSystem.getMemoryNameSystem();
        r.skillManager = skillManager;

        for (const auto& [k, a] : arms)
        {
            r.arms[k] = a.toIce();
        }

        r.kinematicUnitInterface = kinematicUnitInterface;
        r.platformUnitInterface = platformUnitInterface;
        r.localizationUnitInterface = localizationUnitInterface;
        return r;
    }

} // namespace armarx::robot_name_service::core
