#pragma once

#include <map>
#include <memory>
#include <string>
#include <vector>

#include <RobotAPI/interface/robot_name_service/RobotNameServiceInterface.h>
#include <RobotAPI/libraries/armem/client/MemoryNameSystem.h>

namespace armarx::robot_name_service::core
{
    class Hand
    {
    public:
        void fromIce(const armarx::robot_name_service::dto::Hand& r);
        armarx::robot_name_service::dto::Hand toIce() const;

    public:
        std::string name;
        std::string ft_name;
        armarx::HandUnitInterfacePrx handUnitInterface;
    };

    class Arm
    {
    public:
        void fromIce(const armarx::robot_name_service::dto::Arm& r);
        armarx::robot_name_service::dto::Arm toIce() const;

    public:
        std::string kinematicChainName;
        Hand hand;
    };

    class Robot
    {
    public:
        void fromIce(const armarx::robot_name_service::dto::Robot& r);
        armarx::robot_name_service::dto::Robot toIce() const;

    public:
        // header
        std::string name;
        armarx::data::PackagePath xmlPackagePath;

        // memory and skills
        armarx::armem::client::MemoryNameSystem memoryNameSystem;
        armarx::skills::manager::dti::SkillManagerInterfacePrx skillManager;

        // kinematic stuff
        std::map<std::string, Arm> arms;

        // units
        armarx::KinematicUnitInterfacePrx kinematicUnitInterface;
        armarx::PlatformUnitInterfacePrx platformUnitInterface;
        armarx::LocalizationUnitInterfacePrx localizationUnitInterface;
    };
} // namespace armarx::robot_name_service::core
